@@ -1,4 +1,4 @@
-from magiconfig import ArgumentParser, MagiConfigOptions
+from magiconfig import ArgumentParser, MagiConfigOptions, ArgumentDefaultsRawHelpFormatter
 import six
 
 if __name__=="__main__":
@@ -13,10 +13,13 @@ if __name__=="__main__":
     parser = ArgumentParser(
         config_options=MagiConfigOptions(
             strict = True,
-        )
+        ),
+        formatter_class=ArgumentDefaultsRawHelpFormatter,
     )
     parser.add_argument("-v","--verbose", dest="verbose", action="store_true", help="enable verbose output")
     # include schema in parser
-    parser.set_defaults(*config_schema_flat)
+    parser.add_config_only(*config_schema_flat)
+    # include a default value and some required
+    parser.add_config_only(**{"dataset.path": "/data", "dataset.background": None, "dataset.signal": None})
     args = parser.parse_args()
     if args.verbose: six.print_(args)
